@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,27 +32,24 @@
 #include "ComplexGetStatus.h"
 #include "GetterSetterAccessCase.h"
 #include "ICStatusUtils.h"
-#include "InterpreterInlines.h"
 #include "IntrinsicGetterAccessCase.h"
-#include "JSCInlines.h"
-#include "JSScope.h"
-#include "LLIntData.h"
-#include "LowLevelInterpreter.h"
 #include "ModuleNamespaceAccessCase.h"
 #include "PolymorphicAccess.h"
 #include "StructureStubInfo.h"
 #include <wtf/ListDump.h>
 
 namespace JSC {
-namespace DOMJIT {
-class GetterSetter;
-}
 
-bool GetByStatus::appendVariant(const GetByIdVariant& variant)
+bool GetByStatus::appendVariant(const GetByVariant& variant)
 {
     return appendICStatusVariant(m_variants, variant);
 }
 
+void GetByStatus::shrinkToFit()
+{
+    m_variants.shrinkToFit();
+}
+
 GetByStatus GetByStatus::computeFromLLInt(CodeBlock* profiledBlock, BytecodeIndex bytecodeIndex)
 {
     VM& vm = profiledBlock->vm();
@@ -86,6 +83,41 @@ GetByStatus GetByStatus::computeFromLLInt(CodeBlock* profiledBlock, BytecodeInde
     case op_get_by_val:
         return GetByStatus(NoInformation, false);
 
+    case op_iterator_open: {
+        ASSERT(bytecodeIndex.checkpoint() == OpIteratorOpen::getNext);
+        auto& metadata = instruction->as<OpIteratorOpen>().metadata(profiledBlock);
+
+        // FIXME: We should not just bail if we see a get_by_id_proto_load.
+        // https://bugs.webkit.org/show_bug.cgi?id=158039
+        if (metadata.m_modeMetadata.mode != GetByIdMode::Default)
+            return GetByStatus(NoInformation, false);
+        structureID = metadata.m_modeMetadata.defaultMode.structureID;
+        identifier = &vm.propertyNames->next;
+        break;
+    }
+
+    case op_iterator_next: {
+        auto& metadata = instruction->as<OpIteratorNext>().metadata(profiledBlock);
+        if (bytecodeIndex.checkpoint() == OpIteratorNext::getDone) {
+            if (metadata.m_doneModeMetadata.mode != GetByIdMode::Default)
+                return GetByStatus(NoInformation, false);
+            structureID = metadata.m_doneModeMetadata.defaultMode.structureID;
+            identifier = &vm.propertyNames->done;
+        } else {
+            ASSERT(bytecodeIndex.checkpoint() == OpIteratorNext::getValue);
+            if (metadata.m_valueModeMetadata.mode != GetByIdMode::Default)
+                return GetByStatus(NoInformation, false);
+            structureID = metadata.m_valueModeMetadata.defaultMode.structureID;
+            identifier = &vm.propertyNames->value;
+        }
+        break;
+    }
+
+    case op_get_private_name:
+        // FIXME: Consider using LLInt caches or IC information to populate GetByStatus
+        // https://bugs.webkit.org/show_bug.cgi?id=217245
+        return GetByStatus(NoInformation, false);
+
     default: {
         ASSERT_NOT_REACHED();
         return GetByStatus(NoInformation, false);
@@ -108,7 +140,7 @@ GetByStatus GetByStatus::computeFromLLInt(CodeBlock* profiledBlock, BytecodeInde
         return GetByStatus(NoInformation, false);
 
     GetByStatus result(Simple, false);
-    result.appendVariant(GetByIdVariant(nullptr, StructureSet(structure), offset));
+    result.appendVariant(GetByVariant(nullptr, StructureSet(structure), offset));
     return result;
 }
 
@@ -181,13 +213,13 @@ GetByStatus GetByStatus::computeForStubInfoWithoutExitSiteFeedback(
         return GetByStatus(NoInformation);
         
     case CacheType::GetByIdSelf: {
-        Structure* structure = stubInfo->u.byIdSelf.baseObjectStructure.get();
+        Structure* structure = stubInfo->m_inlineAccessBaseStructure.get();
         if (structure->takesSlowPathInDFGForImpureProperty())
             return GetByStatus(JSC::slowVersion(summary), *stubInfo);
-        CacheableIdentifier identifier = stubInfo->getByIdSelfIdentifier();
+        CacheableIdentifier identifier = stubInfo->identifier();
         UniquedStringImpl* uid = identifier.uid();
         RELEASE_ASSERT(uid);
-        GetByIdVariant variant(WTFMove(identifier));
+        GetByVariant variant(WTFMove(identifier));
         unsigned attributes;
         variant.m_offset = structure->getConcurrently(uid, attributes);
         if (!isValidOffset(variant.m_offset))
@@ -231,7 +263,7 @@ GetByStatus GetByStatus::computeForStubInfoWithoutExitSiteFeedback(
             Structure* structure = access.structure();
             if (!structure) {
                 // The null structure cases arise due to array.length and string.length. We have no way
-                // of creating a GetByIdVariant for those, and we don't really have to since the DFG
+                // of creating a GetByVariant for those, and we don't really have to since the DFG
                 // handles those cases in FixupPhase using value profiling. That's a bit awkward - we
                 // shouldn't have to use value profiling to discover something that the AccessCase
                 // could have told us. But, it works well enough. So, our only concern here is to not
@@ -252,7 +284,7 @@ GetByStatus GetByStatus::computeForStubInfoWithoutExitSiteFeedback(
             case ComplexGetStatus::Inlineable: {
                 std::unique_ptr<CallLinkStatus> callLinkStatus;
                 JSFunction* intrinsicFunction = nullptr;
-                FunctionPtr<OperationPtrTag> customAccessorGetter;
+                FunctionPtr<CustomAccessorPtrTag> customAccessorGetter;
                 std::unique_ptr<DOMAttributeAnnotation> domAttribute;
                 bool haveDOMAttribute = false;
 
@@ -290,7 +322,7 @@ GetByStatus GetByStatus::computeForStubInfoWithoutExitSiteFeedback(
                 } }
 
                 ASSERT((AccessCase::Miss == access.type() || access.isCustom()) == (access.offset() == invalidOffset));
-                GetByIdVariant variant(access.identifier(), StructureSet(structure), complexGetStatus.offset(),
+                GetByVariant variant(access.identifier(), StructureSet(structure), complexGetStatus.offset(),
                     complexGetStatus.conditionSet(), WTFMove(callLinkStatus),
                     intrinsicFunction,
                     customAccessorGetter,
@@ -312,6 +344,7 @@ GetByStatus GetByStatus::computeForStubInfoWithoutExitSiteFeedback(
             } }
         }
         
+        result.shrinkToFit();
         return result;
     }
         
@@ -402,10 +435,11 @@ GetByStatus GetByStatus::computeFor(const StructureSet& set, UniquedStringImpl*
         if (attributes & PropertyAttribute::CustomAccessorOrValue)
             return GetByStatus(LikelyTakesSlowPath);
         
-        if (!result.appendVariant(GetByIdVariant(nullptr, structure, offset)))
+        if (!result.appendVariant(GetByVariant(nullptr, structure, offset)))
             return GetByStatus(LikelyTakesSlowPath);
     }
     
+    result.shrinkToFit();
     return result;
 }
 #endif // ENABLE(JIT)
@@ -463,10 +497,11 @@ void GetByStatus::merge(const GetByStatus& other)
         if (m_state != other.m_state)
             return mergeSlow();
         
-        for (const GetByIdVariant& otherVariant : other.m_variants) {
+        for (const GetByVariant& otherVariant : other.m_variants) {
             if (!appendVariant(otherVariant))
                 return mergeSlow();
         }
+        shrinkToFit();
         return;
         
     case ModuleNamespace:
@@ -503,23 +538,30 @@ void GetByStatus::filter(const StructureSet& set)
         m_state = NoInformation;
 }
 
-void GetByStatus::visitAggregate(SlotVisitor& visitor)
+template<typename Visitor>
+void GetByStatus::visitAggregateImpl(Visitor& visitor)
 {
     if (isModuleNamespace())
         m_moduleNamespaceData->m_identifier.visitAggregate(visitor);
-    for (GetByIdVariant& variant : m_variants)
+    for (GetByVariant& variant : m_variants)
         variant.visitAggregate(visitor);
 }
 
-void GetByStatus::markIfCheap(SlotVisitor& visitor)
+DEFINE_VISIT_AGGREGATE(GetByStatus);
+
+template<typename Visitor>
+void GetByStatus::markIfCheap(Visitor& visitor)
 {
-    for (GetByIdVariant& variant : m_variants)
+    for (GetByVariant& variant : m_variants)
         variant.markIfCheap(visitor);
 }
 
+template void GetByStatus::markIfCheap(AbstractSlotVisitor&);
+template void GetByStatus::markIfCheap(SlotVisitor&);
+
 bool GetByStatus::finalize(VM& vm)
 {
-    for (GetByIdVariant& variant : m_variants) {
+    for (GetByVariant& variant : m_variants) {
         if (!variant.finalize(vm))
             return false;
     }
@@ -537,20 +579,7 @@ CacheableIdentifier GetByStatus::singleIdentifier() const
     if (isModuleNamespace())
         return m_moduleNamespaceData->m_identifier;
 
-    if (m_variants.isEmpty())
-        return nullptr;
-
-    CacheableIdentifier result = m_variants.first().identifier();
-    if (!result)
-        return nullptr;
-    for (size_t i = 1; i < m_variants.size(); ++i) {
-        CacheableIdentifier identifier = m_variants[i].identifier();
-        if (!identifier)
-            return nullptr;
-        if (identifier != result)
-            return nullptr;
-    }
-    return result;
+    return singleIdentifierForICStatus(m_variants);
 }
 
 void GetByStatus::dump(PrintStream& out) const
